/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ex;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.sql.Statement;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author jarek
 */
public class EnhancedSQLXMLDemo {

    Connection con = null;
    Statement stmt = null;
    PreparedStatement pstmt = null;
    String tableName = "sample.jtest2";

    public EnhancedSQLXMLDemo() {
        try {
            EnvInfoSingleton env = EnvInfoSingleton.INSTANCE;
            con = DriverManager.getConnection(env.getDbURL() + env.getServerName() + ";trace=" + env.getTrace(), env.getDbUser(),
                    env.getDbPassword());
        } catch (SQLException e) {
            e.printStackTrace();
        }

    // Now we have a valid connection...
    }

    public static void main(String[] args) {
        EnhancedSQLXMLDemo esd = new EnhancedSQLXMLDemo();
        esd.insertSQLXML();
        esd.evaluateXPath("//Sales/Amount/text()", esd.readSQLXMLFromColumn());

    }

    private SQLXML readSQLXMLFromFile(String inputFileName) throws Exception {
        SQLXML sqlXml = con.createSQLXML();

        File binFile = new File(inputFileName);
        InputStream bin = new FileInputStream(binFile);
        int availableLength = bin.available();
        byte[] totalBytes = new byte[availableLength];
        int bytedata = bin.read(totalBytes);
        OutputStream bout = sqlXml.setBinaryStream();
        bout.write(totalBytes);
        if (bin != null) {
            bin.close();
        }
        if (bout != null) {
            bout.close();
        }
        return sqlXml;

    }


    private void insertSQLXML() {
        try {
            stmt = con.createStatement();
            SQLXML sx = con.createSQLXML();
            sx = this.readSQLXMLFromFile("./CountryByRegionSales.xml");
            String psx = "insert into " + tableName +
                    " ( id, xmldata) values(?,?) ";
            do {
                try {
                    pstmt = con.prepareStatement(psx);
                    pstmt.setString(1, "1000");
                    pstmt.setSQLXML(2, sx);
                    pstmt.executeUpdate();
                    break;
                } catch (SQLException rsem) {
                    if (rsem.getSQLState().equalsIgnoreCase("42704")) {
                        System.err.println("Table or view does not exitst, creating...");
                        String crtTable = "CREATE TABLE " + tableName +
                                " (ID VARCHAR(10) CCSID 37 DEFAULT NULL ," +
                                " XMLDATA CLOB(104857600) CCSID 1208 DEFAULT NULL )";
                        try {
                            stmt.execute(crtTable);
                        } catch (SQLException e) {
                            e.printStackTrace();
                        }
                    } else {
                        throw rsem;
                    }
                }
            } while (true);
        } catch (Exception se) {
            se.printStackTrace();
        } finally {
            try {
                if (pstmt != null) {
                    pstmt.close();
                }
                if (stmt != null) {
                    stmt.close();
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private SQLXML readSQLXMLFromColumn() {
        SQLXML sx = null;
        try {
            sx = con.createSQLXML();
            String psx = "SELECT xmldata FROM " + tableName +
                    " WHERE id = ? ";
            pstmt = con.prepareStatement(psx);
            pstmt.setString(1, "1000");
            ResultSet rs = pstmt.executeQuery();
            if (rs.next()) {
                sx = rs.getSQLXML(1);
            }
        } catch (Exception se) {
            se.printStackTrace();
        } finally {
            try {
                if (pstmt != null) {
                    pstmt.close();
                }
                if (con != null) {
                    con.close();
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return sx;
    }

    private void evaluateXPath(String query, SQLXML sx) {
        try {
            InputSource inputSource = new InputSource();
            inputSource.setByteStream(sx.getBinaryStream());
            try {
                XPath xpath = XPathFactory.newInstance().newXPath();
                NodeList nodes = (NodeList) xpath.evaluate(query,
                        inputSource, XPathConstants.NODESET);
                if (nodes == null) {
                    System.out.println("XPath query must return a node-set");
                } else {
                    for (int i = 0; i < nodes.getLength(); i++) {
                        System.out.println("Name: " + nodes.item(i).getNodeName());
                        System.out.println("Value: " + nodes.item(i).getNodeValue());
                    }
                }
            } catch (XPathExpressionException e) {
                System.out.println("Invalid XPath query: " + query);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
